#!/bin/bash
# version 2
host="$1"
sender="$2"

# We want to prevent bounce messages to remote receipients
if [ "$sender" == "" ]; then
        echo -e "KDropping remote bounce message to prevent backscatter.\0"
        exit 0
fi
OLD_DOMAIN=""
DOMAIN="${sender##*@}"
[ "$DKREMOTE" ] || DKREMOTE="/var/qmail/bin/qmail-remote.orig"
[ "$DKSIGN" ]   || DKSIGN="/etc/domainkeys/%/default"
while [ ! -r "${DKSIGN//\%/$DOMAIN}" ] ; do
        if [ "$DOMAIN" == "" -o "$DOMAIN" == "$OLD_DOMAIN" ]; then
          break;
        fi

	# save current $DOMAIN for testing next loop
	OLD_DOMAIN=$DOMAIN

        # try parent domains, per RFC 4871, section 3.8
        DOMAIN=${DOMAIN#*.}
        DPARTS=( ${DOMAIN//./ } )
        [ ${#DPARTS[*]} -eq 1 ] && DOMAIN="${sender##*@}" && break
done
DKSIGN="${DKSIGN//\%/$DOMAIN}"

if [ -r "$DKSIGN" ] ; then
        tmp=`/bin/mktemp -t dk.sign.XXXXXXXXXXXXXXXXXXX`
        tmp2=`/bin/mktemp -t dk2.sign.XXXXXXXXXXXXXXXXXXX`
        /bin/cat - >"$tmp"

        # compute the DomainKey signature
        error=`(/usr/bin/dktest -s "$DKSIGN" -c nofws -r <"$tmp" | \
                /bin/sed 's/; d=.*;/; d='"$DOMAIN"';/' > "$tmp2") 2>&1`
        [ "$error" ] && echo "DomainKey error: $error" >&2 && exit -1

        # feed the signatures and the original message to the real qmail-remote
        /bin/cat "$tmp2" "$tmp" | "$DKREMOTE" "$@"
        retval=$?
        /bin/rm "$tmp" "$tmp2"
        retval=0
        exit $retval
else
        exec "$DKREMOTE" "$@"
fi
